import { EditorManager } from '../base/editor-manager';
/**
 * PasteCleanup for MsWord content
 *
 * @hidden

 */
export declare class MsWordPaste {
    private parent;
    /**
     * Initializes a new instance of the MsWordPaste class
     *
     * @param {EditorManager} parent - The parent editor manager instance
     * @returns {void} - No return value
     */
    constructor(parent?: EditorManager);
    private olData;
    private ulData;
    /** List of HTML node names that should not be ignored during cleanup */
    private ignorableNodes;
    /** List of HTML block node names */
    private blockNode;
    private borderStyle;
    private upperRomanNumber;
    private lowerRomanNumber;
    private lowerGreekNumber;
    private removableElements;
    private listContents;
    private addEventListener;
    private removeEventListener;
    private cropImageDimensions;
    private wordCleanup;
    private addDoubleBr;
    private cleanList;
    private insertAfter;
    private findClosestListElem;
    private addListClass;
    private addTableBorderClass;
    private imageConversion;
    private markUnsupportedImages;
    private isUnsupportedImageShape;
    private extractImageInfo;
    private processHexValues;
    private updateImageSources;
    private cleanUnsupportedImages;
    private checkVShape;
    private processVShapeElement;
    private convertToBase64;
    private conBytesToBase64;
    private conHexStringToBytes;
    private hexConversion;
    private extractImageData;
    private isImageCropped;
    private extractCropValue;
    private removeClassName;
    private breakLineAddition;
    private isReplacableWithBreak;
    private hasNonBreakingSpace;
    private findDetachElem;
    private removeUnwantedElements;
    private removeStyleElements;
    private removeElementsByTagName;
    private findDetachEmptyElem;
    private hasParentWithClass;
    private removeEmptyElements;
    private isEmptyCellWithMsoNormal;
    private isDivWithoutBorder;
    private shouldRemoveEmptyElement;
    private removeEmptyMetaTags;
    private styleCorrection;
    private filterStyles;
    private applyStylesToElements;
    private applyStyleToElementCollection;
    private removeBorderNoneStyles;
    private removeOverlappingStyles;
    private processListStyles;
    private adjustListMargins;
    private adjustMarginLeftValue;
    private removeUnwantedStyle;
    private findStyleObject;
    private cleanupStyleText;
    private removingComments;
    private listCleanUp;
    private shouldIgnoreElement;
    private isMsoListParagraph;
    private isFirstListItem;
    private shouldAddListSeparator;
    private isBlockElement;
    /**
     * Converts MS Word list nodes to standard HTML lists
     *
     * @param {Element[]} listNodes - Array of list nodes to convert
     * @returns {void} - No return value
     * @private
     */
    private listConverter;
    private processListNodes;
    private fixOutlineLevel;
    private extractNestingLevel;
    private extractListFormatOverride;
    private determineListProperties;
    private determineStartAttribute;
    private updateNodeStyle;
    private replaceNodesWithLists;
    private getlistStyleType;
    private getOrderedListStyleType;
    private getUnorderedListStyleType;
    private makeConversion;
    private isStandardListType;
    private shouldResetListItem;
    private createParagraphWithContent;
    private isNewRootList;
    private createRootList;
    private isSameLevelList;
    private addToSameLevelList;
    private isDeeperNestedList;
    private createNestedList;
    private createStandardNestedList;
    private getLastListItem;
    private isTopLevelList;
    private handleTopLevelList;
    private handleOtherNestingScenarios;
    private createDifferentFormatList;
    private applyListItemStyles;
    private setStartAttributeIfNeeded;
    private getListContent;
    private isImageList;
    private handleImageList;
    private handleTextList;
    private cleanupListIgnoreTags;
    private cleanupListOrder;
    private processListOrderElement;
    private extractBulletMarker;
    private processMargin;
    private processListItemMargins;
    private processTableMargins;
    private processIgnoredNodeMargins;
    private removeEmptyAnchorTag1;
    private removeEmptyAnchorTag;
    private findSource;
    private handleOneNoteContent;
    /**
     * Cleans up resources when the component is destroyed
     *
     * @returns {void} - No return value
     * @public
     */
    destroy(): void;
}
